//public packages
import 'dart:core';
import 'package:args/args.dart';
import 'package:http/http.dart' as http;
import 'dart:convert';

//My stuff
import 'domains/sudoku/SudokuProblem.dart';

late ArgResults argResults;
String help =
    'Usage: sudoku [OPTION]...\nA simple dart program to generate and display a sudoku game.\n\n  -h, --help          Display this message\n  -i, --initialHints  number of givens on starting board, defaulting to 30';

void main(List<String> arguments) async {
  final parser = ArgParser()
    ..addOption('initialHints',
        abbr: 'i', help: 'number of givens on board', defaultsTo: '17')
    ..addFlag('help', abbr: 'h')
    ..addOption('count', abbr: 'n', defaultsTo: '1')
    ..addOption('url', abbr: 'u', defaultsTo: 'sudoku.rusondia.net')
    ..addOption('scheme', abbr: 's', defaultsTo: 'https')
    ..addOption('port', abbr: 'p', defaultsTo: '443');

  argResults = parser.parse(arguments);

  if (argResults['help']) {
    print(help);
  } else {
    var hints = int.parse(argResults['initialHints']);
    var count = int.parse(argResults['count']);
    var port = int.parse(argResults['port']);
    var scheme = argResults['scheme'];
    var url = argResults['url'];

    var uri = Uri(
        scheme: scheme,
        host: url,
        port: port,
        path: '/api/v1/hints/$hints/count/$count');

    var problems = await getProblemsFromUri(uri);
    for (var problem in problems) {
      print('Initial board');
      print(SudokuProblem.stateToString(problem.getInitialState()));
      print('\nFinal Board');
      print(SudokuProblem.stateToString(problem.getFinalState()));
    }
  }
}

Future<List<dynamic>> _call(Uri uri) async {
  var response = await http.get(uri);
  var json = jsonDecode(response.body);
  var boards = json['boards'];
  return boards;
}

Future<List<SudokuProblem>> getProblemsFromUri(Uri uri) async {
  var boards = await _call(uri);
  var problems = <SudokuProblem>[];
  for (var board in boards) {
    problems.add(SudokuProblem.fromJSON(board));
  }
  return problems;
}

Future<List<SudokuProblem>> getProblems(hints, count) async {
  var uri = Uri(
      scheme: 'https',
      host: 'sudoku.rusondia.net',
      port: 443,
      path: '/api/v1/hints/$hints/count/$count');
  var boards = await _call(uri);
  var problems = <SudokuProblem>[];
  for (var board in boards) {
    problems.add(SudokuProblem.fromJSON(board));
  }
  return problems;
}
